@@ -1,21 +1,20 @@
-# Copyright (c) 2003-2016 LOGILAB S.A. (Paris, FRANCE).
-# http://www.logilab.fr/ -- mailto:contact@logilab.fr
+# Copyright (c) 2006, 2009-2010, 2012-2015 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
+# Copyright (c) 2014-2016 Claudiu Popa <pcmanticore@gmail.com>
+
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
 
 """check for signs of poor design"""
 
-import re
 from collections import defaultdict
 
 from astroid import If, BoolOp
+from astroid import decorators
 
 from pylint.interfaces import IAstroidChecker
 from pylint.checkers import BaseChecker
 from pylint.checkers.utils import check_messages
-
-# regexp for ignored argument name
-IGNORED_ARGUMENT_NAMES = re.compile('_.*')
+from pylint import utils
 
 
 MSGS = {
@@ -93,12 +92,6 @@ class MisdesignChecker(BaseChecker):
                 {'default' : 5, 'type' : 'int', 'metavar' : '<int>',
                  'help': 'Maximum number of arguments for function / method'}
                ),
-               ('ignored-argument-names',
-                {'default' : IGNORED_ARGUMENT_NAMES,
-                 'type' :'regexp', 'metavar' : '<regexp>',
-                 'help' : 'Argument names that match this expression will be '
-                          'ignored. Default to name with leading underscore'}
-               ),
                ('max-locals',
                 {'default' : 15, 'type' : 'int', 'metavar' : '<int>',
                  'help': 'Maximum number of locals for function / method body'}
@@ -166,19 +159,20 @@ class MisdesignChecker(BaseChecker):
         self._returns = []
         self._branches = defaultdict(int)
 
+    @decorators.cachedproperty
+    def _ignored_argument_names(self):
+        return utils.get_global_option(self, 'ignored-argument-names', default=None)
+
     @check_messages('too-many-ancestors', 'too-many-instance-attributes',
                     'too-few-public-methods', 'too-many-public-methods')
     def visit_classdef(self, node):
         """check size of inheritance hierarchy and number of instance attributes
         """
-        # Is the total inheritance hierarchy is 7 or less?
         nb_parents = len(list(node.ancestors()))
         if nb_parents > self.config.max_parents:
             self.add_message('too-many-ancestors', node=node,
                              args=(nb_parents, self.config.max_parents))
-        # Does the class contain less than 20 attributes for
-        # non-GUI classes (40 for GUI)?
-        # FIXME detect gui classes
+
         if len(node.instance_attrs) > self.config.max_attributes:
             self.add_message('too-many-instance-attributes', node=node,
                              args=(len(node.instance_attrs),
@@ -226,12 +220,14 @@ class MisdesignChecker(BaseChecker):
         self._returns.append(0)
         # check number of arguments
         args = node.args.args
+        ignored_argument_names = self._ignored_argument_names
         if args is not None:
-            ignored_args_num = len(
-                [arg for arg in args
-                 if self.config.ignored_argument_names.match(arg.name)])
+            ignored_args_num = 0
+            if ignored_argument_names:
+                ignored_args_num = sum(1 for arg in args if ignored_argument_names.match(arg.name))
+
             argnum = len(args) - ignored_args_num
-            if  argnum > self.config.max_args:
+            if argnum > self.config.max_args:
                 self.add_message('too-many-arguments', node=node,
                                  args=(len(args), self.config.max_args))
         else:
