@@ -31,10 +31,11 @@ from pylint.utils import get_global_option
 from pylint.checkers import BaseChecker
 from pylint.checkers import utils
 
+
 SPECIAL_OBJ = re.compile("^_{2}[a-z]+_{2}$")
 FUTURE = '__future__'
 # regexp for ignored argument name
-IGNORED_ARGUMENT_NAMES = re.compile('_.*')
+IGNORED_ARGUMENT_NAMES = re.compile('_.*|^ignored_|^unused_')
 PY3K = sys.version_info >= (3, 0)
 
 
@@ -336,7 +337,7 @@ class VariablesChecker(BaseChecker):
                  'help' : 'Tells whether we should check for unused import in '
                           '__init__ files.'}),
                ("dummy-variables-rgx",
-                {'default': ('_+$|(_[a-zA-Z0-9_]*[a-zA-Z0-9]+?$)|dummy'),
+                {'default': '_+$|(_[a-zA-Z0-9_]*[a-zA-Z0-9]+?$)|dummy|^ignored_|^unused_',
                  'type' :'regexp', 'metavar' : '<regexp>',
                  'help' : 'A regular expression matching the name of dummy '
                           'variables (i.e. expectedly not used).'}),
@@ -865,6 +866,7 @@ class VariablesChecker(BaseChecker):
             in_annotation_or_default = (
                 (
                     PY3K and (node in frame.args.annotations
+                              or node in frame.args.kwonlyargs_annotations
                               or node is frame.args.varargannotation
                               or node is frame.args.kwargannotation)
                 )
@@ -891,6 +893,7 @@ class VariablesChecker(BaseChecker):
                                recursive_klass):
         maybee0601 = True
         annotation_return = False
+        use_outer_definition = False
         if frame is not defframe:
             maybee0601 = _detect_global_scope(node, frame, defframe)
         elif defframe.parent is None:
@@ -905,6 +908,10 @@ class VariablesChecker(BaseChecker):
             forbid_lookup = isinstance(frame, astroid.FunctionDef) and _assigned_locally(node)
             if not forbid_lookup and defframe.root().lookup(name)[1]:
                 maybee0601 = False
+                use_outer_definition = (
+                    stmt == defstmt
+                    and not isinstance(defnode, astroid.node_classes.Comprehension)
+                )
             else:
                 # check if we have a nonlocal
                 if name in defframe.locals:
@@ -960,7 +967,7 @@ class VariablesChecker(BaseChecker):
                     # same line as the function definition
                     maybee0601 = False
 
-        return maybee0601, annotation_return
+        return maybee0601, annotation_return, use_outer_definition
 
     def _ignore_class_scope(self, node, name, frame):
         # Detect if we are in a local class scope, as an assignment.
@@ -1032,9 +1039,7 @@ class VariablesChecker(BaseChecker):
                 self._loopvar_name(node, name)
                 break
             found_node = self._next_to_consume(node, name, to_consume)
-            if found_node:
-                consumed[name] = found_node
-            else:
+            if found_node is None:
                 continue
             # checks for use before assignment
             defnode = utils.assign_parent(to_consume[name][0])
@@ -1048,11 +1053,14 @@ class VariablesChecker(BaseChecker):
                                    isinstance(defframe, astroid.ClassDef) and
                                    node.name == defframe.name)
 
-                maybee0601, annotation_return = self._is_variable_violation(
+                maybee0601, annotation_return, use_outer_definition = self._is_variable_violation(
                     node, name, defnode, stmt, defstmt,
                     frame, defframe,
                     base_scope_type, recursive_klass)
 
+                if use_outer_definition:
+                    continue
+
                 if (maybee0601
                         and not utils.is_defined_before(node)
                         and not astroid.are_exclusive(stmt, defstmt, ('NameError',))):
@@ -1094,6 +1102,7 @@ class VariablesChecker(BaseChecker):
                             self.add_message('undefined-variable',
                                              node=node, args=name)
 
+            consumed[name] = found_node
             del to_consume[name]
             # check it's not a loop variable used outside the loop
             self._loopvar_name(node, name)
@@ -1266,7 +1275,7 @@ class VariablesChecker3k(VariablesChecker):
 
     def _check_classdef_metaclasses(self, klass, parent_node):
         if not klass._metaclass:
-            # Skip if this class doesn't use explictly a metaclass, but inherits it from ancestors
+            # Skip if this class doesn't use explicitly a metaclass, but inherits it from ancestors
             return []
 
         consumed = []  # [(scope_locals, consumed_key)]
