@@ -1,7 +1,7 @@
 """ Checks that classes uses valid __slots__ """
 
 # pylint: disable=too-few-public-methods, missing-docstring, no-absolute-import
-# pylint: disable=using-constant-test
+# pylint: disable=using-constant-test, wrong-import-position, no-else-return
 from collections import deque
 
 def func():
@@ -28,15 +28,12 @@ class FourthGood(object):
     __slots__ = ('a%s' % i for i in range(10))
 
 class FifthGood(object):
-    __slots__ = "a"
-
-class SixthGood(object):
     __slots__ = deque(["a", "b", "c"])
 
-class SeventhGood(object):
+class SixthGood(object):
     __slots__ = {"a": "b", "c": "d"}
 
-class Bad(object):
+class Bad(object): # [invalid-slots]
     __slots__ = list
 
 class SecondBad(object):  # [invalid-slots]
@@ -51,11 +48,38 @@ class FourthBad(object):  # [invalid-slots]
 class FifthBad(object):
     __slots__ = ("a", "b", "")  # [invalid-slots-object]
 
+class SixthBad(object):  # [single-string-used-for-slots]
+    __slots__ = "a"
+
+class SeventhBad(object):  # [single-string-used-for-slots]
+    __slots__ = ('foo')
+
+class EighthBad(object):  # [single-string-used-for-slots]
+    __slots__ = deque.__name__
+
 class PotentiallyGood(object):
     __slots__ = func()
 
 class PotentiallySecondGood(object):
     __slots__ = ('a', deque.__name__)
 
+
+import six
+
+
+class Metaclass(type):
+
+    def __iter__(cls):
+        for value in range(10):
+            yield str(value)
+
+
+@six.add_metaclass(Metaclass)
+class IterableClass(object):
+    pass
+
 class PotentiallyThirdGood(object):
-    __slots__ = deque.__name__
+    __slots__ = IterableClass
+
+class PotentiallyFourthGood(object):
+    __slots__ = Good.__slots__
